@@ -18,11 +18,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-import optparse
-import re
-import stat
-import sys
-import time
 from optparse import make_option
 import logging
 
@@ -30,8 +25,7 @@ import jhbuild.moduleset
 import jhbuild.frontends
 from jhbuild.errors import UsageError, FatalError, CommandError
 from jhbuild.commands import Command, BuildCommand, register_command
-
-from jhbuild.config import parse_relative_time
+from jhbuild.utils import uprint, N_, _
 
 
 class cmd_update(Command):
@@ -100,7 +94,7 @@ class cmd_updateone(Command):
         module_set = jhbuild.moduleset.load(config)
         try:
             module_list = [module_set.get_module(modname, ignore_case = True) for modname in args]
-        except KeyError, e:
+        except KeyError as e:
             raise FatalError(_("A module called '%s' could not be found.") % e)
 
         if not module_list:
@@ -138,7 +132,7 @@ class cmd_cleanone(Command):
         module_set = jhbuild.moduleset.load(config)
         try:
             module_list = [module_set.get_module(modname, ignore_case = True) for modname in args]
-        except KeyError, e:
+        except KeyError as e:
             raise FatalError(_("A module called '%s' could not be found.") % e)
 
         if not module_list:
@@ -234,8 +228,8 @@ class cmd_build(BuildCommand):
 
         module_set = jhbuild.moduleset.load(config)
         modules = args or config.modules
-        full_module_list = module_set.get_full_module_list \
-                               (modules, config.skip,
+        full_module_list = module_set.get_full_module_list(
+                                modules, config.skip,
                                 include_suggests=not config.ignore_suggests,
                                 include_afters=options.build_optional_modules)
         full_module_list = module_set.remove_tag_modules(full_module_list,
@@ -325,7 +319,7 @@ class cmd_buildone(BuildCommand):
             modname = modname.rstrip(os.sep)
             try:
                 module = module_set.get_module(modname, ignore_case=True)
-            except KeyError, e:
+            except KeyError:
                 default_repo = jhbuild.moduleset.get_default_repo()
                 if not default_repo:
                     continue
@@ -369,7 +363,7 @@ class cmd_run(Command):
             return self.run(config, options, args)
         try:
             return os.execlp(args[0], *args)
-        except OSError, exc:
+        except OSError as exc:
             raise FatalError(_("Unable to execute the command '%(command)s': %(err)s") % {
                     'command':args[0], 'err':str(exc)})
 
@@ -379,7 +373,7 @@ class cmd_run(Command):
             module_set = jhbuild.moduleset.load(config)
             try:
                 module = module_set.get_module(module_name, ignore_case = True)
-            except KeyError, e:
+            except KeyError as e:
                 raise FatalError(_("A module called '%s' could not be found.") % e)
 
             build = jhbuild.frontends.get_buildscript(config, [module], module_set=module_set)
@@ -389,7 +383,7 @@ class cmd_run(Command):
                 workingdir = module.get_srcdir(build)
             try:
                 build.execute(args, cwd=workingdir)
-            except CommandError, exc:
+            except CommandError as exc:
                 if args:
                     raise FatalError(_("Unable to execute the command '%s'") % args[0])
                 else:
@@ -399,7 +393,7 @@ class cmd_run(Command):
                 os.execlp(args[0], *args)
             except IndexError:
                 raise FatalError(_('No command given'))
-            except OSError, exc:
+            except OSError as exc:
                 raise FatalError(_("Unable to execute the command '%(command)s': %(err)s") % {
                         'command':args[0], 'err':str(exc)})
 
