@@ -17,35 +17,37 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from __future__ import print_function
+
 import os
 import sys 
 import logging
 import shlex
 import subprocess
-import pipes
-import imp
 import textwrap
 import time
-from StringIO import StringIO
 import re
 
-import cmds
+from .compat import TextIO
+from . import cmds
+from . import _, udecode
 
 def get_installed_pkgconfigs(config):
     """Returns a dictionary mapping pkg-config names to their current versions on the system."""
     pkgversions = {}
     try:
         proc = subprocess.Popen(['pkg-config', '--list-all'], stdout=subprocess.PIPE, close_fds=True)
-        stdout = proc.communicate()[0]
+        stdout = udecode(proc.communicate()[0])
         proc.wait()
         pkgs = []
-        for line in StringIO(stdout):
+        for line in TextIO(stdout):
             pkg, rest = line.split(None, 1)
             pkgs.append(pkg)
 
         # see if we can get the versions "the easy way"
         try:
             stdout = subprocess.check_output(['pkg-config', '--modversion'] + pkgs)
+            stdout = udecode(stdout)
             versions = stdout.splitlines()
             if len(versions) == len(pkgs):
                 return dict(zip(pkgs, versions))
@@ -60,6 +62,7 @@ def get_installed_pkgconfigs(config):
             args.append(pkg)
             proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
             stdout = proc.communicate()[0]
+            stdout = udecode(stdout)
             proc.wait()
             pkgversions[pkg] = stdout.strip()
     except (subprocess.CalledProcessError, OSError): # pkg-config not installed
@@ -116,18 +119,18 @@ def systemdependencies_met(module_name, sysdeps, config):
                 shell_split = shlex.split
             try:
                 while True:
-                    arg = itr.next()
+                    arg = next(itr)
                     if arg.strip() in ['-I', '-isystem']:
                         # extract paths handling quotes and multiple paths
-                        paths += shell_split(itr.next())[0].split(os.pathsep)
+                        paths += shell_split(next(itr))[0].split(os.pathsep)
                     elif arg.startswith('-I'):
                         paths += shell_split(arg[2:])[0].split(os.pathsep)
             except StopIteration:
                 pass
             return paths
         try:
-            multiarch = subprocess.check_output(['gcc', '-print-multiarch']).strip()
-        except:
+            multiarch = udecode(subprocess.check_output(['gcc', '-print-multiarch'])).strip()
+        except (EnvironmentError, subprocess.CalledProcessError):
             multiarch = None
         # search /usr/include and its multiarch subdir (if any) by default
         paths = [ os.path.join(os.sep, 'usr', 'include')]
@@ -139,10 +142,10 @@ def systemdependencies_met(module_name, sysdeps, config):
         paths += extract_path_from_cflags(os.environ.get('CXXFLAGS', ''))
         # check include paths incorrectly configured in makeargs
         paths += extract_path_from_cflags(config.makeargs)
-        paths += extract_path_from_cflags(config.module_autogenargs.get
-                                             (module_name, ''))
-        paths += extract_path_from_cflags(config.module_makeargs.get
-                                             (module_name, ''))
+        paths += extract_path_from_cflags(config.module_autogenargs.get(
+            module_name, ''))
+        paths += extract_path_from_cflags(config.module_makeargs.get(
+            module_name, ''))
         paths += os.environ.get('C_INCLUDE_PATH', '').split(':')
         paths += os.environ.get('CPLUS_INCLUDE_PATH', '').split(':')
         paths = list(set(paths)) # remove duplicates
@@ -176,14 +179,9 @@ def systemdependencies_met(module_name, sysdeps, config):
             if not found:
                 dep_met = False
 
-        elif dep_type == 'python2':
-            try:
-                imp.find_module(value)
-            except:
-                dep_met = False
-
-        elif dep_type == 'python3':
-            python3_script = textwrap.dedent('''
+        elif dep_type in ('python2', 'python3'):
+            command = dep_type
+            python_script = textwrap.dedent('''
                 import imp
                 import sys
                 try:
@@ -192,7 +190,7 @@ def systemdependencies_met(module_name, sysdeps, config):
                     exit(1)
                 ''').strip('\n')
             try:
-                subprocess.check_call(['python3', '-c', python3_script, value])
+                subprocess.check_call([command, '-c', python_script, value])
             except (subprocess.CalledProcessError, OSError):
                 dep_met = False
 
@@ -208,8 +206,7 @@ def systemdependencies_met(module_name, sysdeps, config):
             try:
                 # no xmlcatalog installed will (correctly) fail the check
                 subprocess.check_output(['xmlcatalog', xml_catalog, value])
-
-            except:
+            except (EnvironmentError, subprocess.CalledProcessError):
                 dep_met = False
 
         # check alternative dependencies
@@ -268,17 +265,14 @@ class PKSystemInstall(SystemInstall):
     def _get_new_transaction(self):
         if self._loop is None:
             try:
-                import glib
-            except:
-                try:
-                    from gi.repository import GLib as glib
-                except:
-                    raise SystemExit(_('Error: python-gobject package not found.'))
-            self._loop = glib.MainLoop()
+                from gi.repository import GLib
+            except ImportError:
+                raise SystemExit(_('Error: python-gobject package not found.'))
+            self._loop = GLib.MainLoop()
         if self._sysbus is None:
             try:
                 import dbus.glib
-            except:
+            except ImportError:
                 raise SystemExit(_('Error: dbus-python package not found.'))
             import dbus
             self._dbus = dbus
@@ -287,7 +281,6 @@ class PKSystemInstall(SystemInstall):
             self._pkdbus = dbus.Interface(self._sysbus.get_object('org.freedesktop.PackageKit',
                                                                   '/org/freedesktop/PackageKit'),
                                           'org.freedesktop.PackageKit')
-            properties = dbus.Interface(self._pkdbus, 'org.freedesktop.DBus.Properties')
         txn_path = self._pkdbus.CreateTransaction()
         txn = self._sysbus.get_object('org.freedesktop.PackageKit', txn_path)
         txn_tx = self._dbus.Interface(txn, 'org.freedesktop.PackageKit.Transaction')
@@ -386,7 +379,7 @@ class PacmanSystemInstall(SystemInstall):
 
         for name, filename in uninstalled_filenames:
             try:
-                result = subprocess.check_output(['pkgfile', '--raw', filename])
+                result = udecode(subprocess.check_output(['pkgfile', '--raw', filename]))
                 if result:
                     package_names.add(result.split('\n')[0])
             except subprocess.CalledProcessError:
@@ -412,10 +405,13 @@ class AptSystemInstall(SystemInstall):
     def __init__(self):
         SystemInstall.__init__(self)
 
-    def _apt_file_result_pkgconfig(self):
-        apt_file_result = subprocess.check_output(["apt-file", "search", "--regex", "\\.pc$"])
+    def _apt_file_result(self, regexp):
+        if regexp is None or regexp == "":
+            raise RuntimeError("regexp mustn't be None or empty")
+        apt_file_result = subprocess.check_output(["apt-file", "search", "--regexp", regexp])
+        apt_file_result = udecode(apt_file_result)
         ret_value = []
-        for line in StringIO(apt_file_result):
+        for line in TextIO(apt_file_result):
             parts = line.split(':', 1)
             if len(parts) != 2:
                 continue
@@ -424,56 +420,70 @@ class AptSystemInstall(SystemInstall):
             ret_value.append((name, path))
         return ret_value
 
-    def _apt_file_result_regexp(self, paths):
-        regexp = self._build_apt_file_path_regexp(paths)
-        if regexp is None:
-            return []
-        apt_file_result = subprocess.check_output(["apt-file", "search", "--regex", regexp])
-        ret_value = []
-        for line in StringIO(apt_file_result):
-            parts = line.split(':', 1)
-            if len(parts) != 2:
-                continue
-            name = parts[0]
-            path = parts[1].strip()
-            ret_value.append((name, path))
-        return ret_value
-
-    def _build_apt_file_path_regexp(self, paths):
+    def _build_apt_file_path_regexp_exact(self, paths):
         if len(paths) == 0:
             return None
         first_path = paths[0][1]
+        if first_path.startswith("/"):
+            first_path = first_path[1:]
         ret_value = re.escape(first_path)
-        for modname, path in paths:
+        for modname, path in paths[1:]:
             if path.startswith("/"):
                 path = path[1:]
             ret_value += "|" + re.escape(path)
         return ret_value
 
-    def _name_match_pkg(self, pkg, apt_file_result, native_packages):
-        for name, path in apt_file_result:
-            if path.endswith(pkg):
-                native_packages.append(name)
-                return True
-        return False
+    def _build_apt_file_path_regexp_pc_or_c_includes(self, paths):
+        if len(paths) == 0:
+            return None
+        first_path = paths[0][1]
+        if first_path.startswith("/"):
+            first_path = first_path[1:]
+        ret_value = re.escape(first_path)
+        for modname, path in paths[1:]:
+            if path.startswith("/"):
+                path = path[1:]
+            ret_value += "|.*/" + re.escape(path)
+        return ret_value
 
-    def _name_match_exact(self, binary, apt_file_result, native_packages):
+    def _name_match_exact(self, exact_path_to_match, apt_file_result, native_packages):
         for name, path in apt_file_result:
-            if path == binary:
+            if path == exact_path_to_match:
                 native_packages.append(name)
                 return True
         return False
 
     def _append_native_packages_or_warn_pkgconfig(self, pkgconfigs, native_packages):
-        apt_file_result = self._apt_file_result_pkgconfig()
+        if len(pkgconfigs) == 0:
+            return
+
+        def get_pkg_config_search_paths():
+            output = subprocess.check_output(
+                ["pkg-config", "--variable", "pc_path", "pkg-config"])
+            output = udecode(output)
+            return output.strip().split(os.pathsep)
+
+        # Various packages include zlib.pc (emscripten, mingw) so look only in
+        # the default pkg-config search paths
+        search_paths = get_pkg_config_search_paths()
+        search_paths = tuple(os.path.join(p, "") for p in search_paths)
+
+        apt_file_result = self._apt_file_result(regexp="\\.pc$")
         for modname, pkg in pkgconfigs:
-            if not self._name_match_pkg(pkg, apt_file_result, native_packages):
+            for name, path in apt_file_result:
+                if path.endswith("/" + pkg) and path.startswith(search_paths):
+                    native_packages.append(name)
+                    break
+            else:
                 logging.info(_('No native package found for %(id)s '
                                '(%(filename)s)') % {'id'       : modname,
                                                     'filename' : pkg})
 
     def _append_native_packages_or_warn_exact(self, paths, native_packages):
-        apt_file_result = self._apt_file_result_regexp(paths)
+        if len(paths) == 0:
+            return
+        exact_regexp = self._build_apt_file_path_regexp_exact(paths)
+        apt_file_result = self._apt_file_result(exact_regexp)
         for modname, path in paths:
             if not self._name_match_exact(path, apt_file_result, native_packages):
                 logging.info(_('No native package found for %(id)s '
@@ -481,10 +491,13 @@ class AptSystemInstall(SystemInstall):
                                                     'filename' : path})
 
     def _append_native_packages_or_warn_c_includes(self, c_includes, native_packages, multiarch):
-        apt_file_result = self._apt_file_result_regexp(c_includes)
+        if len(c_includes) == 0:
+            return
+        c_includes_regexp = self._build_apt_file_path_regexp_pc_or_c_includes(c_includes)
+        apt_file_result = self._apt_file_result(c_includes_regexp)
         for modname, filename in c_includes:
             # Try multiarch first, so we print the non-multiarch location on failure.
-            if (multiarch == None or
+            if (multiarch is None or
                     not self._name_match_exact('/usr/include/%s/%s' % (multiarch, filename), apt_file_result, native_packages)):
                 if not self._name_match_exact('/usr/include/%s' % filename, apt_file_result, native_packages):
                     logging.info(_('No native package found for %(id)s '
@@ -505,7 +518,7 @@ class AptSystemInstall(SystemInstall):
         logging.info(_('Using apt-file to search for providers; this may be extremely slow. Please wait. Patience!'))
         native_packages = []
 
-        pkgconfigs = [(modname, '/%s.pc' % pkg) for modname, pkg in
+        pkgconfigs = [(modname, '%s.pc' % pkg) for modname, pkg in
                       get_uninstalled_pkgconfigs(uninstalled)]
         self._append_native_packages_or_warn_pkgconfig(pkgconfigs, native_packages)
 
@@ -516,11 +529,11 @@ class AptSystemInstall(SystemInstall):
         # Get multiarch include directory, e.g. /usr/include/x86_64-linux-gnu
         multiarch = None
         try:
-            multiarch = subprocess.check_output(['gcc', '-print-multiarch']).strip()
-        except:
+            multiarch = udecode(subprocess.check_output(['gcc', '-print-multiarch'])).strip()
+        except (EnvironmentError, subprocess.CalledProcessError):
             # Really need GCC to continue. Yes, this is fragile.
             self._install_packages(['gcc'])
-            multiarch = subprocess.check_output(['gcc', '-print-multiarch']).strip()
+            multiarch = udecode(subprocess.check_output(['gcc', '-print-multiarch'])).strip()
 
         c_includes = get_uninstalled_c_includes(uninstalled)
         self._append_native_packages_or_warn_c_includes(c_includes, native_packages, multiarch)
@@ -540,5 +553,5 @@ _classes = [AptSystemInstall, PacmanSystemInstall, PKSystemInstall]
 if __name__ == '__main__':
     logging.basicConfig(level=logging.INFO)
     installer = SystemInstall.find_best()
-    print "Using %r" % (installer, )
+    print("Using %r" % (installer, ))
     installer.install(sys.argv[1:])
