@@ -22,13 +22,11 @@ __all__ = []
 __metaclass__ = type
 
 import os
-import sys
-import urlparse
 from subprocess import Popen, PIPE
 
 from jhbuild.errors import FatalError, CommandError
 from jhbuild.versioncontrol import Repository, Branch, register_repo_type
-from jhbuild.commands.sanitycheck import inpath
+from jhbuild.utils import inpath, _, urlutils
 
 class HgRepository(Repository):
     """A class representing a Mercurial repository.
@@ -51,13 +49,13 @@ class HgRepository(Repository):
             module = self.config.branches[name]
             if not module:
                 raise FatalError(_('branch for %(name)s has wrong override, check your %(filename)s') % \
-                                   {'name'     : name,
-                                    'filename' : self.config.filename})
+                                 {'name'     : name,
+                                  'filename' : self.config.filename})
         else:
             if module is None:
                 module = name
             if not self.href.startswith('ssh://'):
-                module = urlparse.urljoin(self.href, module)
+                module = urlutils.urljoin(self.href, module)
             else:
                 module = self.href + module
         return HgBranch(self, module, checkoutdir)
@@ -69,17 +67,17 @@ class HgRepository(Repository):
 class HgBranch(Branch):
     """A class representing a Mercurial branch."""
 
+    @property
     def srcdir(self):
         if self.checkoutdir:
             return os.path.join(self.checkoutroot, self.checkoutdir)
         else:
             return os.path.join(self.checkoutroot,
                                 os.path.basename(self.module))
-    srcdir = property(srcdir)
 
+    @property
     def branchname(self):
         return None
-    branchname = property(branchname)
 
     def _checkout(self, buildscript):
         cmd = ['hg', 'clone', self.module]
@@ -111,7 +109,7 @@ class HgBranch(Branch):
         try:
             hg = Popen(['hg', 'ti', '--template', '{node}'], stdout=PIPE,
                        cwd=self.srcdir)
-        except OSError, e:
+        except OSError as e:
             raise CommandError(str(e))
         return hg.stdout.read().strip()
 
